proc start_step { step } {
  set stopFile ".stop.rst"
  if {[file isfile .stop.rst]} {
    puts ""
    puts "*** Halting run - EA reset detected ***"
    puts ""
    puts ""
    return -code error
  }
  set beginFile ".$step.begin.rst"
  set platform "$::tcl_platform(platform)"
  set user "$::tcl_platform(user)"
  set pid [pid]
  set host ""
  if { [string equal $platform unix] } {
    if { [info exist ::env(HOSTNAME)] } {
      set host $::env(HOSTNAME)
    }
  } else {
    if { [info exist ::env(COMPUTERNAME)] } {
      set host $::env(COMPUTERNAME)
    }
  }
  set ch [open $beginFile w]
  puts $ch "<?xml version=\"1.0\"?>"
  puts $ch "<ProcessHandle Version=\"1\" Minor=\"0\">"
  puts $ch "    <Process Command=\".planAhead.\" Owner=\"$user\" Host=\"$host\" Pid=\"$pid\">"
  puts $ch "    </Process>"
  puts $ch "</ProcessHandle>"
  close $ch
}

proc end_step { step } {
  set endFile ".$step.end.rst"
  set ch [open $endFile w]
  close $ch
}

proc step_failed { step } {
  set endFile ".$step.error.rst"
  set ch [open $endFile w]
  close $ch
}

set_msg_config -id {HDL 9-1061} -limit 100000
set_msg_config -id {HDL 9-1654} -limit 100000

start_step init_design
set ACTIVE_STEP init_design
set rc [catch {
  create_msg_db init_design.pb
  set_property design_mode GateLvl [current_fileset]
  set_param project.singleFileAddWarning.threshold 0
  set_property webtalk.parent_dir c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.cache/wt [current_project]
  set_property parent.project_path c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.xpr [current_project]
  set_property ip_output_repo c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.cache/ip [current_project]
  set_property ip_cache_permissions {read write} [current_project]
  set_property XPM_LIBRARIES {XPM_CDC XPM_MEMORY} [current_project]
  add_files -quiet c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.runs/synth_1/xilinx_pcie_2_1_ep_7x.dcp
  add_files -quiet c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.srcs/sources_1/ip/pcie_7x_0/pcie_7x_0.dcp
  set_property netlist_only true [get_files c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.srcs/sources_1/ip/pcie_7x_0/pcie_7x_0.dcp]
  read_xdc -ref pcie_7x_0 -cells inst c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.srcs/sources_1/ip/pcie_7x_0/source/pcie_7x_0-PCIE_X0Y0.xdc
  set_property processing_order EARLY [get_files c:/Avnet/MMP_BB2_pcie/pcie_7x_0_ex/pcie_7x_0_ex.srcs/sources_1/ip/pcie_7x_0/source/pcie_7x_0-PCIE_X0Y0.xdc]
  read_xdc C:/Avnet/MMP_BB2_pcie/xilinx_pcie_7x_ep_x4g2.xdc
  link_design -top xilinx_pcie_2_1_ep_7x -part xc7z100ffg900-2
  write_hwdef -file xilinx_pcie_2_1_ep_7x.hwdef
  close_msg_db -file init_design.pb
} RESULT]
if {$rc} {
  step_failed init_design
  return -code error $RESULT
} else {
  end_step init_design
  unset ACTIVE_STEP 
}

start_step opt_design
set ACTIVE_STEP opt_design
set rc [catch {
  create_msg_db opt_design.pb
  opt_design 
  write_checkpoint -force xilinx_pcie_2_1_ep_7x_opt.dcp
  catch { report_drc -file xilinx_pcie_2_1_ep_7x_drc_opted.rpt }
  close_msg_db -file opt_design.pb
} RESULT]
if {$rc} {
  step_failed opt_design
  return -code error $RESULT
} else {
  end_step opt_design
  unset ACTIVE_STEP 
}

start_step place_design
set ACTIVE_STEP place_design
set rc [catch {
  create_msg_db place_design.pb
  implement_debug_core 
  place_design 
  write_checkpoint -force xilinx_pcie_2_1_ep_7x_placed.dcp
  catch { report_io -file xilinx_pcie_2_1_ep_7x_io_placed.rpt }
  catch { report_utilization -file xilinx_pcie_2_1_ep_7x_utilization_placed.rpt -pb xilinx_pcie_2_1_ep_7x_utilization_placed.pb }
  catch { report_control_sets -verbose -file xilinx_pcie_2_1_ep_7x_control_sets_placed.rpt }
  close_msg_db -file place_design.pb
} RESULT]
if {$rc} {
  step_failed place_design
  return -code error $RESULT
} else {
  end_step place_design
  unset ACTIVE_STEP 
}

start_step route_design
set ACTIVE_STEP route_design
set rc [catch {
  create_msg_db route_design.pb
  route_design 
  write_checkpoint -force xilinx_pcie_2_1_ep_7x_routed.dcp
  catch { report_drc -file xilinx_pcie_2_1_ep_7x_drc_routed.rpt -pb xilinx_pcie_2_1_ep_7x_drc_routed.pb -rpx xilinx_pcie_2_1_ep_7x_drc_routed.rpx }
  catch { report_methodology -file xilinx_pcie_2_1_ep_7x_methodology_drc_routed.rpt -rpx xilinx_pcie_2_1_ep_7x_methodology_drc_routed.rpx }
  catch { report_timing_summary -warn_on_violation -max_paths 10 -file xilinx_pcie_2_1_ep_7x_timing_summary_routed.rpt -rpx xilinx_pcie_2_1_ep_7x_timing_summary_routed.rpx }
  catch { report_power -file xilinx_pcie_2_1_ep_7x_power_routed.rpt -pb xilinx_pcie_2_1_ep_7x_power_summary_routed.pb -rpx xilinx_pcie_2_1_ep_7x_power_routed.rpx }
  catch { report_route_status -file xilinx_pcie_2_1_ep_7x_route_status.rpt -pb xilinx_pcie_2_1_ep_7x_route_status.pb }
  catch { report_clock_utilization -file xilinx_pcie_2_1_ep_7x_clock_utilization_routed.rpt }
  close_msg_db -file route_design.pb
} RESULT]
if {$rc} {
  write_checkpoint -force xilinx_pcie_2_1_ep_7x_routed_error.dcp
  step_failed route_design
  return -code error $RESULT
} else {
  end_step route_design
  unset ACTIVE_STEP 
}

start_step write_bitstream
set ACTIVE_STEP write_bitstream
set rc [catch {
  create_msg_db write_bitstream.pb
  set_property XPM_LIBRARIES {XPM_CDC XPM_MEMORY} [current_project]
  catch { write_mem_info -force xilinx_pcie_2_1_ep_7x.mmi }
  write_bitstream -force -no_partial_bitfile xilinx_pcie_2_1_ep_7x.bit 
  catch { write_sysdef -hwdef xilinx_pcie_2_1_ep_7x.hwdef -bitfile xilinx_pcie_2_1_ep_7x.bit -meminfo xilinx_pcie_2_1_ep_7x.mmi -file xilinx_pcie_2_1_ep_7x.sysdef }
  catch {write_debug_probes -quiet -force debug_nets}
  close_msg_db -file write_bitstream.pb
} RESULT]
if {$rc} {
  step_failed write_bitstream
  return -code error $RESULT
} else {
  end_step write_bitstream
  unset ACTIVE_STEP 
}

